## 5.2節

library(XML) # XMLパッケージの読込み

# XMLパッケージに付属のサンプルファイルを使用
fileName <- system.file("exampleData", "job.xml", package="XML")

# 1. XML文書をパース
objXML <- xmlParse(fileName)
# 2. 名前空間の定義を取得
vec.namespaces <- xmlNamespaceDefinitions(objXML,simplify=TRUE)
# 3. 目的のノードを指定→ 要素ノード：gjob:Application を指定
nodes.application <- getNodeSet(objXML,path="//gjob:Application",namespaces=vec.namespaces)
# 4. ノードの属性または内容の抽出→ 内容を抽出
value <- xmlSApply(nodes.application,FUN=xmlValue)
## 結果を確認する
value # gjob:Application 要素の内容が取得できたはず

## 5.3節

## 必要に応じて，任意のフォルダをWorking Directoryに指定
# setwd("F:/R/EDINET")

library("RCurl")
library("XML")
library("plyr")

api.service.url <- "http://resource.ufocatch.com/atom/edinetx" # APIのURI
strSIC <- "1301" # 対象企業の証券コードをクエリワードにする

# リクエストを送信する
strURL <- paste0(api.service.url,"/query/",strSIC,sep="") # URIと証券コードを結合
objQuery <- httpGET(strURL) # レスポンスを取得

# xmlParseにより，APIのResponseをXMLInternalDocument形式に変換
objXML <- xmlParse(objQuery,encoding="UTF-8")
# 名前空間の定義をベクトル形式(simplify=TRUE) で取得
objXML.namespaces <- xmlNamespaceDefinitions(objXML,simplify=TRUE)
# デフォルトの名前空間に適当な接頭辞を付け直す
names(objXML.namespaces)[ names(objXML.namespaces)=="" ] <- "default"

# 要素ノード：entryを指定
nodes.entry <- getNodeSet(objXML,"//default:entry",namespaces=objXML.namespaces)
# 有価証券報告書のみを抽出
lst.YUHO <- list()
for(node in nodes.entry){ # <entry>タグをひとつずつ処理
  lst.temp <- list()
  # 提出書類のタイトルを取得→要素ノード：title の内容を取得
  title.value <- xpathSApply(node,path="default:title",fun=xmlValue,namespaces=objXML.namespaces)
  # 提出書類のタイトルに「有価証券報告書」を含むか否かを判定
  is.YUHO <- grepl(pat="*有価証券報告書*",x=title.value)
  if(is.YUHO){ #「含む」と判定された場合
    # IDを取得
    lst.temp$id <- xpathSApply(node,path="default:id",fun=xmlValue,namespaces=objXML.namespaces)
    lst.temp$title <- title.value
    # <link>タグのうち，type='application/zip'のhref属性を取得
    lst.temp$url <- xpathSApply(node,path="default:link[@type='application/zip']/@href",namespaces=objXML.namespaces)
    lst.YUHO[[lst.temp$id]] <- lst.temp
  }
}

# XBRL形式のファイル一式をダウンロード，"data"フォルダに保存
dir.create("data") # "data"フォルダを作成
for(lst in lst.YUHO){
  temp <- getBinaryURL(url=lst$url ) # バイナリデータ
  writeBin( temp, paste0("data/",lst$id,".zip") ) # zip形式で保存
}

# 抽出した情報をplyr パッケージのldply 関数でデータフレームに変換
dat.downloaded <- ldply(lst.YUHO,.fun=data.frame)[, -1]

# CSV形式で保存
write.csv(x=dat.downloaded,file="downloaded_XBRL.csv",row.names=FALSE)


## 5.4節

## 必要に応じて，任意のフォルダをWorking Directoryに指定
## Working Directory配下に，2013年度版EDINETタクソノミ：サンプルインスタンスを展開
setwd("F:/R/EDINET")

library(XBRL)

# 2013年版EDINETタクソノミ：サンプルインスタンス
xbrl.dir <- "sample/サンプルインスタンス/X99001-000(1計算書方式)/"
xbrl.path <- paste0(xbrl.dir,"jpfr-asr-X99001-000-2013-03-31-01-2013-06-27.xbrl")

# XBRLパッケージにより報告書インスタンスを処理
xbrl.parsed <- xbrlDoAll(xbrl.path)

# 金額の定義および、文書情報の定義を抽出
dat.facts <- xbrl.parsed$fact
# 内容(=facts) の文字コードを，UTF-8 からShift-JIS に修正
dat.facts$fact <- iconv(x=dat.facts$fact,from="UTF-8",to="Shift-JIS")

# 名称リンクの情報を抽出
dat.labels <- xbrl.parsed$label
dat.labels$labelString <- iconv(x=dat.labels$labelString,from="UTF-8",to="Shift-JIS")
# 日本語(lang=="ja")かつ，ドキュメンテーション用(labelRole=…/role/documentation) 
dat.labels <- subset(dat.labels,lang=="ja" & labelRole=="http://www.xbrl.org/2003/role/documentation",select=c("elementId","labelString"))
dat.labels <- unique(dat.labels) # 重複を消去

# 金額・文書情報の定義と名称リンクを結合
dat.FI <- merge(x=dat.facts,y=dat.labels,by="elementId")

# 当期・連結＝contextIdが"CurrentYearConsolidated"から始まる要素
target <- grepl(pattern="^CurrentYearConsolidated",x=dat.FI$contextId)
dat.FI.CurrentYearConsolidated <- dat.FI[target,]
head(dat.FI.CurrentYearConsolidated[,c("labelString","fact")])

# 文書情報の定義のみ抽出
target <- grepl(pattern="^DocumentInfo",x=dat.FI$contextId)
dat.Presenter.Info <- dat.FI[target,]
# 開示対象者の名称
print( subset( dat.Presenter.Info, elementId == "jpfr-di_EntityNameJaEntityInformation" )$fact )

# コンテキストの定義
dat.contexts <- xbrl.parsed$context
# 期首・期末
print( subset( dat.contexts, contextId == "CurrentYearConsolidatedDuration")$startDate )
print( subset( dat.contexts, contextId == "CurrentYearConsolidatedDuration")$endDate )


### 5.4.3 報告書インスタンスの処理：発展的な方法 ###

# 表示リンクの情報を抽出
dat.ps <- xbrl.parsed$presentation
# 表示リンクの情報のうち，連結貸借対照表に関係する情報を抽出
dat.ps.CYC <- subset(dat.ps,subset=grepl(pattern="*role/ConsolidatedBalanceSheets$",x=roleId))

# 「流動資産」に対応する要素名を確認
subset( dat.labels,subset=grepl(pattern="^流動資産*",x=labelString) )

# 子孫要素を再帰的に抽出する関数
gather.descendant <- function(df, parent){
  children <- as.vector( df$toElementId[df$fromElementId==parent] ) # 直接の子を抽出
  return( c(children,unlist( sapply(children,function(child){ gather.descendant(df,child) }) ) ) ) # 子の直接の子を再帰的に抽出
}

# jpfr-t-cte_CurrentAssetsAbstract の子孫要素を取得
vec.descendants <- gather.descendant(dat.ps.CYC,parent="jpfr-t-cte_CurrentAssetsAbstract")

# 当期・連結ベースの財務諸表から，「流動資産」の子孫要素のみを抽出
dat.FI.CYC.CAA <- subset(dat.FI.CurrentYearConsolidated,subset=(elementId %in% vec.descendants) )
dat.FI.CYC.CAA[,c("labelString","fact")]